<?php

/*
 * @copyright   2016 Mautic, Inc. All rights reserved
 * @author      Mautic, Inc
 *
 * @link        https://mautic.org
 *
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace MauticPlugin\MauticUProcBundle\Controller;

use Mautic\FormBundle\Controller\FormController;
use Mautic\LeadBundle\Entity\Company;
use Mautic\LeadBundle\Entity\Lead;
use Mautic\UserBundle\Entity\User;
use Mautic\UserBundle\Model\UserModel;
use Mautic\UserBundle\Model\LeadModel;
use Symfony\Component\HttpFoundation\Response;
use Mautic\LeadBundle\Entity\DoNotContact;

class PublicController extends FormController
{
    /**
     * Write a notification.
     *
     * @param string    $message   Message of the notification
     * @param string    $header    Header for message
     * @param string    $iconClass Font Awesome CSS class for the icon (e.g. fa-eye)
     * @param User|null $user      User object; defaults to current user
     */
    public function addNewNotification($message, $header, $iconClass, User $user)
    {
        /** @var \Mautic\CoreBundle\Model\NotificationModel $notificationModel */
        $notificationModel = $this->getModel('core.notification');
        $notificationModel->addNotification($message, 'UProc', false, $header, $iconClass, null, $user);
    }

    private function updateCompany($logger, $message) {

    }

    private function updatePerson($logger, $message) {

    }

    /**
     * @return Response
     *
     * @throws \InvalidArgumentException
     */
    public function callbackAction()
    {
        $logger = $this->get('monolog.logger.mautic');

        if (!$this->request->request->has('result')
            || !$this->request->request->has('message')
            || !$this->request->query->get('id')//oid
            || "completed" !== $this->request->request->get('status')
        ) {
            $logger->log('error', 'ERROR on UProc callback: Malformed request variables: '.json_encode($this->request->request->all(), JSON_PRETTY_PRINT));

            return new Response('ERROR');
        }

        /** @var array $result */
        $result           = $this->request->request->get('result');
        $message           = $this->request->request->get('message');
        $params           = $this->request->request->get('params');
        $oid              = urldecode($this->request->query->get('id'));

        $logger->log('error', json_encode($result, JSON_PRETTY_PRINT));
        $logger->log('error', json_encode($message, JSON_PRETTY_PRINT));
        //$logger->log('error', json_encode($oid, JSON_PRETTY_PRINT));
        $logger->log('error', json_encode($params, JSON_PRETTY_PRINT));

        $type = isset($params['email']) ? "person": "company";
        $validatedRequest = $this->get('mautic.plugin.uproc.lookup_helper')->validateRequest($oid, $type);

        if (!$validatedRequest) {
            $logger->log('error', 'ERROR on UProc callback: Wrong body or id in request: id='.$oid.' body='.json_encode($message, JSON_PRETTY_PRINT));

            return new Response('ERROR');
        }

        $notify = $validatedRequest['notify'];

        try {
            if ('person' === $type) {
                //$logger->log('error', $type);
              /** @var \Mautic\LeadBundle\Model\LeadModel $model */
                $model = $this->getModel('lead');
                /** @var Lead $lead */
                $lead       = $validatedRequest['entity'];
                $currFields = $lead->getFields(true);
                $logger->log('debug', 'CURRFIELDS: '.var_export($currFields, true));

                $loc = [];
                /*
                if (array_key_exists('geo', $result)) {
                    $loc = $result['geo'];
                }*/

                $data = [];
                //$logger->log('error', 'contact: '.var_export($message['contact'], true));

                //For email verification callback. Response = undeliverable is dangerous
                if (array_key_exists('response', $message)) {
                  if ($message['response'] === 'undeliverable' && strpos($message['response'], 'unknown') === false) {
                    $model->addDncForLead($lead, 'email', 'Marked as bounce by UProc email verifier', DoNotContact::MANUAL);
                  }
                } else {
                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'gender',
                          $message['contact']
                      )
                      && empty($currFields['title']['value']) && !empty($message['contact']['gender'])
                  ) {
                      $data['title'] = $message['contact']['gender'] === 'male' ? 'Mr' : 'Mrs';
                  }

                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'surname',
                          $message['contact']
                      )
                      && empty($currFields['lastname']['value'])
                  ) {
                      $data['lastname'] = $message['contact']['surname'];
                  }

                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'name',
                          $message['contact']
                      )
                      && empty($currFields['firstname']['value'])
                  ) {
                      $data['firstname'] = $message['contact']['name'];
                  }

                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'company',
                          $message['contact']
                      )
                      && empty($currFields['company']['value'])
                  ) {
                      $data['company'] = $message['contact']['company'];
                  }

                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'role',
                          $message['contact']
                      )
                      && empty($currFields['position']['value'])
                  ) {
                      $data['position'] = $message['contact']['role'];
                  }

                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'phone',
                          $message['contact']
                      )
                      && empty($currFields['phone']['value'])
                  ) {
                      $data['phone'] = strpos($message['contact']['phone'], ",") !== false ? explode(",", $message['contact']['phone'])[0] : $message['contact']['phone'];
                  }

                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'website',
                          $message['contact']
                      )
                      && empty($currFields['website']['value'])
                  ) {
                      $data['website'] = $message['contact']['website'];
                  }

                  foreach(array('instagram', 'linkedin', 'googleplus', 'facebook', 'twitter') as $v) {
                    $key = $v === 'googleplus' ? 'google' : $v;
                    if (array_key_exists('social', $message)
                        && array_key_exists(
                            $key,
                            $message['social']
                        )
                        && empty($currFields[$v]['value'])
                    ) {
                        $data[$v] = $message['social'][$key] ? $message['social'][$key] : "";
                    }
                  }

                  //Personal profile
                  if (array_key_exists('contact', $message)
                      && array_key_exists(
                          'profile',
                          $message['contact']
                      )
                      && empty($currFields['linkedin']['value'])
                  ) {
                      $data['linkedin'] = $message['contact']['profile'];
                  }

                  if (array_key_exists('location', $message)
                      && array_key_exists(
                          'address',
                          $message['location']
                      )
                      && empty($currFields['address1']['value'])
                  ) {
                      $data['address1'] = $message['location']['address'];
                  }

                  if (array_key_exists('location', $message)
                      && array_key_exists(
                          'zipcode',
                          $message['location']
                      )
                      && empty($currFields['zipcode']['value'])
                  ) {
                      $data['zipcode'] = $message['location']['zipcode'];
                  }

                  if (array_key_exists('location', $message)
                      && array_key_exists(
                          'city',
                          $message['location']
                      )
                      && empty($currFields['city']['value'])
                  ) {
                      $data['city'] = $message['location']['city'];
                  }

                  if (array_key_exists('location', $message)
                      && array_key_exists(
                          'county',
                          $message['location']
                      )
                      && empty($currFields['county']['value'])
                  ) {
                      $data['county'] = $message['location']['county'];
                  }

                  if (array_key_exists('location', $message)
                      && array_key_exists(
                          'state',
                          $message['location']
                      )
                      && empty($currFields['state']['value'])
                  ) {
                      $data['state'] = $message['location']['state'];
                  }

                  if (array_key_exists('location', $message)
                      && array_key_exists(
                          'country',
                          $message['location']
                      )
                      && empty($currFields['country']['value'])
                  ) {
                      $data['country'] = $message['location']['country'];
                  }

                  $logger->log('debug', 'SETTING FIELDS: '.print_r($data, true));

                  // Unset the nonce so that it's not used again
                  $socialCache = $lead->getSocialCache();
                  unset($socialCache['uproc']['nonce']);
                  $lead->setSocialCache($socialCache);

                  $model->setFieldValues($lead, $data);
                  $model->saveEntity($lead);
                }

                if ($notify && (!isset($lead->imported) || !$lead->imported)) {
                    /** @var UserModel $userModel */
                    $userModel = $this->getModel('user');
                    if ($user = $userModel->getEntity($notify)) {
                        $this->addNewNotification(
                            sprintf($this->translator->trans('mautic.plugin.uproc.contact_retrieved'), $lead->getEmail()),
                            'UProc Plugin',
                            'fa-search',
                            $user
                        );
                    }
                }
            } else {
                /******************  COMPANY STUFF  *********************/

                if ('company' === $type) {
                    /** @var \Mautic\LeadBundle\Model\CompanyModel $model */
                    $model = $this->getModel('lead.company');
                    /** @var Company $company */
                    $company    = $validatedRequest['entity'];
                    $currFields = $company->getFields(true);

                    if (array_key_exists('location', $message)
                        && array_key_exists(
                            'address',
                            $message['location']
                        )
                        && empty($currFields['companyaddress1']['value'])
                    ) {
                        $data['companyaddress1'] = $message['location']['address'];
                    }

                    if (array_key_exists('location', $message)
                        && array_key_exists(
                            'zipcode',
                            $message['location']
                        )
                        && empty($currFields['companyzipcode']['value'])
                    ) {
                        $data['companyzipcode'] = $message['location']['zipcode'];
                    }

                    if (array_key_exists('location', $message)
                        && array_key_exists(
                            'country',
                            $message['location']
                        )
                        && empty($currFields['companycountry']['value'])
                    ) {
                        $data['companycountry'] = $message['location']['country'];
                    }

                    if (array_key_exists('location', $message)
                        && array_key_exists(
                            'city',
                            $message['location']
                        )
                        && empty($currFields['companycity']['value'])
                    ) {
                        $data['companycity'] = $message['location']['city'];
                    }

                    if (array_key_exists('location', $message)
                        && array_key_exists(
                            'state',
                            $message['location']
                        )
                        && empty($currFields['companystate']['value'])
                    ) {
                        $data['companystate'] = $message['location']['state'];
                    }

                    if (array_key_exists('contact', $message)
                        && array_key_exists(
                            'name',
                            $message['contact']
                        )
                        && empty($currFields['companyname']['value'])
                    ) {
                        $data['companyname'] = $message['contact']['name'];
                    }

                    if (array_key_exists('contact', $message)
                        && array_key_exists(
                            'phone',
                            $message['contact']
                        )
                        && empty($currFields['companyphone']['value'])
                    ) {
                        $data['companyphone'] = $message['contact']['phone'];
                    }

                    if (array_key_exists('contact', $message)
                        && array_key_exists(
                            'website',
                            $message['contact']
                        )
                        && empty($currFields['companywebsite']['value'])
                    ) {
                        $data['companywebsite'] = $message['contact']['website'];
                    }

                    /*

                    if (array_key_exists('city', $loc) && empty($currFields['companycity']['value'])) {
                        $data['companycity'] = $loc['city'];
                    }

                    if (array_key_exists('metrics', $result)
                        && array_key_exists(
                            'employees',
                            $result['metrics']
                        )
                        && empty($currFields['companynumber_of_employees']['value'])
                    ) {
                        $data['companynumber_of_employees'] = $result['metrics']['employees'];
                    }

                    if (array_key_exists('description', $result) && empty($currFields['companydescription']['value'])) {
                        $data['companydescription'] = $result['description'];
                    }

                    if (array_key_exists('phone', $result) && empty($currFields['companyphone']['value'])) {
                        $data['companyphone'] = $result['phone'];
                    }

                    if (array_key_exists('site', $result)
                        && array_key_exists(
                            'emailAddresses',
                            $result['site']
                        )
                        && count($result['site']['emailAddresses'])
                        && empty($currFields['companyemail']['value'])
                    ) {
                        $data['companyemail'] = $result['site']['emailAddresses'][0];
                    }

                    if (array_key_exists('country', $loc) && empty($currFields['companycountry']['value'])) {
                        $data['companycountry'] = $loc['country'];
                    }

                    if (array_key_exists('postalCode', $loc) && empty($currFields['companyzipcode']['value'])) {
                        $loc['postalCode'];
                    }

                    if (array_key_exists('state', $loc) && empty($currFields['companystate']['value'])) {
                        $data['companystate'] = $loc['state'];
                    }*/

                    $logger->log('debug', 'SETTING FIELDS: '.print_r($data, true));

                    // Unset the nonce so that it's not used again
                    $socialCache = $company->getSocialCache();
                    unset($socialCache['uproc']['nonce']);
                    $company->setSocialCache($socialCache);

                    $model->setFieldValues($company, $data);
                    $model->saveEntity($company);

                    if ($notify) {
                        /** @var UserModel $userModel */
                        $userModel = $this->getModel('user');
                        if ($user = $userModel->getEntity($notify)) {
                            $this->addNewNotification(
                                sprintf($this->translator->trans('mautic.plugin.uproc.company_retrieved'), $company->getName()),
                                'UProc Plugin',
                                'fa-search',
                                $user
                            );
                        }
                    }
                }
            }
        } catch (\Exception $ex) {
            $logger->log('error', 'ERROR on UProc callback: '.$ex->getMessage());
            try {
                if ($notify) {
                    /** @var UserModel $userModel */
                    $userModel = $this->getModel('user');
                    if ($user = $userModel->getEntity($notify)) {
                        $this->addNewNotification(
                            sprintf(
                                $this->translator->trans('mautic.plugin.uproc.unable'),
                                $ex->getMessage()
                            ),
                            'UProc Plugin',
                            'fa-exclamation',
                            $user
                        );
                    }
                }
            } catch (\Exception $ex2) {
                $this->get('monolog.logger.mautic')->log('error', 'UProc: '.$ex2->getMessage());
            }
        }

        return new Response('OK');
    }
}
